#module  LIB_SLEEPX  "V1.0"

/*|++LIBSLEEP.C

 Facility: 
	Fermilab Accelerator Control System - VAX/VMS ACNET

 Abstract:
	Routines to wait (sleep) for a specified time.

 Environment:
	General library of additional runtime routines.
|--*/
/*
 Modification History:
 Author: F. Nagy

 V1.0	01-Aug-85  FJN	Created to replace LIBSLEEP.FOR (re-entrant) and
			to change from $SCHDWK/$HIBER to $SETIMR/$WAITFR
			to improve modularity.
*/

/******************
 * Include Files: *
 ******************/
#include  vaxtypes			/* VAX-11 C Type extensions */

/********************************
 * Local definitions and types: *
 ********************************/
typedef unsigned long status;		/* Condition code status type */

					/* System time units... */
#define	TICKS_PER_SECOND	(-10*1000*1000)
#define	TICKS_PER_MILLISECOND	(-10*1000)

/**********************************************
 * Global Procedure and Function definitions: *
 **********************************************/
status LIB$GET_EF(), LIB$FREE_EF();	/* Event flag management routines */
status SYS$SETIMR();			/* Set timer system service */
status SYS$WAITFR();			/* Wait for event flag service */
status LIB$EMUL();			/* Extended Multiply instruction */


/*+ LIB_SLEEP
 Put process to sleep until an absolute time or delay process
 execution for a specified amount of time.

  status.wlc.v = LIB_SLEEP( time.rq.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 time      quadword containing system-format time or delta-time
           which specifies the delay (or sleep-until) time.

 Note: the delay is implemented using $SETIMR to set an event flag
 and $WAITFR to wait for the event flag.  The event flag is allocated
 by using LIB$GET_EF (and released after the $WAITFR returns).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_SLEEP( delay)
vmstime delay;				/* Delay time in system format */
{
register status sts;			/* Temporary for status */
int efn;				/* Event flag number */

/*
 Allocate and event flag for internal use.
*/
sts = LIB$GET_EF( &efn);
RETURN_ON_FAILURE( sts);

/*
 Set timer to SET event flag when the timer expires after the delay time.
*/
sts = SYS$SETIMR( efn, delay, 0, 0);
RETURN_ON_FAILURE( sts);

/*
 Wait for timer to expire - process now goes to sleep...
*/
sts = SYS$WAITFR( efn);
RETURN_ON_FAILURE( sts);

/*
 Sleep period over, release the event flag and exit.
*/
return LIB$FREE_EF( &efn);
}


/*+ _SLEEP_SECONDS
 Put process to sleep for a specified number of seconds.

  status.wlc.v = LIB_SLEEP_SECONDS( seconds.rlu.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 seconds   unsigned quantity giving the number of seconds to delay
           before continuing execution.

 Note: the delay is implemented using $SETIMR to set an event flag
 and $WAITFR to wait for the event flag.  The event flag is allocated
 by using LIB$GET_EF (and released after the $WAITFR returns).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_SLEEP_SECONDS( seconds)
int *seconds;				/* Number of seconds to sleep */
{
register status sts;			/* Temporary for status */
vmstime delay;				/* To get delay in system units */

/*
 Convert sleep time in seconds to sleep time delay in system units.
*/
sts = LIB$EMUL( seconds, &TICKS_PER_SECOND, &0, delay);
RETURN_ON_FAILURE( sts);

/*
 Sleep until delay expires...
*/
return LIB_SLEEP( delay);
}


/*+ _SLEEP_MILLISEC
 Put process to sleep for a specified number of milliseconds.

  status.wlc.v = LIB_SLEEP_MILLISECS( msecs.rlu.r )

 status    completion status (returned in R0) from the $SCHDWK
           system service.

 msecs     unsigned quantity giving the number of milliseconds to
           delay before continuing execution.

 Note: the delay is implemented using $SETIMR to set an event flag
 and $WAITFR to wait for the event flag.  The event flag is allocated
 by using LIB$GET_EF (and released after the $WAITFR returns).

 Due to other system activity and the clock granularity, the length of
 time that the process actually waits may be a bit longer than what
 was specified by the call argument.
-*/

status LIB_SLEEP_MILLISECS( msecs)
int *msecs;				/* Number of milliseconds to sleep */
{
register status sts;			/* Temporary for status */
vmstime delay;				/* To get delay in system units */

/*
 Convert sleep time in milliseconds to sleep time delay in system units.
*/
sts = LIB$EMUL( msecs, &TICKS_PER_MILLISECOND, &0, delay);
RETURN_ON_FAILURE( sts);

/*
 Sleep until delay expires...
*/
return LIB_SLEEP( delay);
}
